from advance_adb.adb import Adb
from advance_adb.files import files
from advance_adb.app import app
import os
class utils:
    def __init__(self, device_id=None):
        self.adb = Adb(device_id)
        self.file = files(device_id)
        self.app = app(device_id)
        
    def manual_debloat(self):
        '''Manually debloats the device'''
        print("Fetching list of packages installed in your device...")
        packages = self.app.get_installed_packages_names()
        launchable_packages = []
        for p in packages:
            if self.app.if_launchable(p):
                launchable_packages.append(p)
        print("List of packages installed in your device :")
        for i in range(len(launchable_packages)):
            print(f'{i + 1}. {launchable_packages[i]}')
        apps = input("Enter apps which you want to remove (like 3 or 1,3,5) : ")
        l = apps.split(",")
        for i in l:
            if i.isdigit():
                print(f"Uninstalling {launchable_packages[int(i) - 1]}...")
                result = self.adb.shell(f'pm uninstall --user 0 {launchable_packages[int(i) - 1]}')
                if "Success" in result:
                    print(f"Successfully uninstalled {launchable_packages[int(i) - 1]}.")
                else:
                    print(f"Failed to uninstall {launchable_packages[int(i) - 1]}. Error: {result}")
            else:
                print("Wrong input")
                break
        print("Debloat successful.")
    def backup(self):
        '''Backs up all the picture, videos, documents and audio file into separated directories'''
        self.backup_pictures()
        self.backup_videos()
        self.backup_documents()
        self.backup_audio()
        print("Backup successful.")
    def backup_pictures(self):
        '''Backs up all the pictures from across the device into specified path'''
        self.adb.shell('cd /sdcard && find -type f \\( -name "*.jpg" -o -name "*.jpeg" -o -name "*.png" \\) > jpeg_files.txt')
        self.file.pull('/sdcard/jpeg_files.txt', 'jpeg_files.txt')
        self.adb.shell('rm /sdcard/jpeg_files.txt')
        with open('jpeg_files.txt', 'r') as f:
            for line in f.readlines():
                line = line.strip()
                line = line.strip('./')
                line = './sdcard/' + line
                self.file.pull(line, line.split('/')[-1])
        os.system('mkdir -p Pictures')
        os.system('mv *.jpg Pictures')
        os.system('mv *.jpeg Pictures')
        os.system('mv *.png Pictures')
        print("Pictures backed up successfully.")
    def backup_videos(self):
        self.adb.shell('cd /sdcard && find -type f \\( -name "*.mp4" -o -name "*.mkv" -o -name "*.mpeg" \\) > video_files.txt')
        self.file.pull('/sdcard/video_files.txt', 'video_files.txt')
        self.adb.shell('rm /sdcard/video_files.txt')
        with open('video_files.txt', 'r') as f:
            for line in f.readlines():
                line = line.strip()
                line = line.strip('./')
                line = './sdcard/' + line
                self.file.pull(line, line.split('/')[-1])
        os.system('mkdir -p Videos')
        os.system('mv *.mp4 Videos')
        os.system('mv *.mkv Videos')
        os.system('mv *.mpeg Videos')
        print("Videos backed up successfully.")
    def backup_documents(self):
        self.adb.shell('cd /sdcard && find -type f \\( -name "*.pdf" -o -name "*.docx" \\) > doc_files.txt')
        self.file.pull('/sdcard/doc_files.txt', 'doc_files.txt')
        self.adb.shell('rm /sdcard/doc_files.txt')
        with open('doc_files.txt', 'r') as f:
            for line in f.readlines():
                line = line.strip()
                line = line.strip('./')
                line = './sdcard/' + line
                self.file.pull(line, line.split('/')[-1])
        os.system('mkdir -p Documents')
        os.system('mv *.pdf Documents')
        os.system('mv *.docx Documents')
        print("Documents backed up successfully.")
    def backup_audio(self):
        self.adb.shell('cd /sdcard && find -type f \\( -name "*.mp3" -o -name "*.wav" \\) > audio_files.txt')
        self.file.pull('/sdcard/audio_files.txt', 'audio_files.txt')
        self.adb.shell('rm /sdcard/audio_files.txt')
        with open('audio_files.txt', 'r') as f:
            for line in f.readlines():
                line = line.strip()
                line = line.strip('./')
                line = './sdcard/' + line
                self.file.pull(line, line.split('/')[-1])
        os.system('mkdir -p Audio')
        os.system('mv *.wav Audio')
        os.system('mv *.mp3 Audio')
        print("Audio backed up successfully.")
    def backup_apps(self):
        '''Backs up all the apps (with or without data) into specified path'''
        self.adb.shell_raw('backup -all -apk -shared -f backup.ab')
        print("Backup successful.")
    def restore(self):
        '''Copies file from system to device'''
        self.file.push('/Pictures/*', '/sdcard/Pictures/')
        self.file.push('/Videos/*', '/sdcard/Videos/')
        self.file.push('/Documents/*', '/sdcard/Documents/')
        self.file.push('/Audio/*', '/sdcard/Audio/')
        print("Files restored successfully.")
    def restore_apps(self):
        '''Restores apps (with or without data) to device'''
        self.adb.shell_raw(f'restore backup.ab')
        print("Restore successful.")


from advance_adb.adb import Adb
from advance_adb.files import files
from advance_adb.app import app
from advance_adb.misc import Misc
from time import sleep
import csv

class whatsapp:
    def __init__(self, device_id=None):
        self.adb = Adb(device_id)
        self.files = files()
        self.app = app()
        self.misc = Misc()
        
    def __csv_to_dict(self, file_path):
        '''Converts the csv file to dictionary'''
        with open(file_path, 'r') as file:
            reader = csv.reader(file)
            num_msg_dict = dict(reader)
        return num_msg_dict
    def send_msg(self, mob_no, msg):
        '''Sends the specified whatsapp message to specified/multiple mobile number'''
        if type(mob_no) == str:
            mob_no = [mob_no]
        for mob in mob_no:
            self.adb.shell(f'am start -a android.intent.action.VIEW -d "whatsapp://send?phone={mob}&text={msg}"')
            sleep(5)
            self.adb.shell('input keyevent 66')
        self.app.force_stop('com.whatsapp')
    def send_bulk_msg(self, num_msg_dict):
        '''Sends the specified whatsapp message to specified multiple mobile numbers
        \nnum_msg_dict: dictionary or csv file containing mobile number and message'''
        if type(num_msg_dict) != dict:
            num_msg_dict = self.__csv_to_dict(num_msg_dict)
        for mob, msg in num_msg_dict.items():
            self.send_msg(mob, msg)
        sleep(2)
        self.app.force_stop('com.whatsapp')


from advance_adb.adb import Adb
from advance_adb.files import files

class Analytics:
    def __init__(self, device_id=None):
        self.adb = Adb(device_id)
        self.file = files(device_id)
        
    @staticmethod
    def __parse_data_to_dic(data):
        id = 1
        final_dic = {}
        data = data.split('\n')
        data = [d[7:] for d in data]
        for d in data:
            pairs = d.split(', ')
            d = {}
            for p in pairs:
                p = p.split('=')
                if len(p)>1:
                    d.update({p[0]:p[1]})
            final_dic.update({id:d})
            id += 1
        return final_dic
    
    def retrieve_call_log(self):
        return Analytics.__parse_data_to_dic(self.adb.shell('content query --uri content://call_log/calls'))
    
    def analyse_call_log(self):
        '''Analyse the call log and return a dictionary with the following keys:
        - 5 most frequent caller
        - 5 most longest call duration'''
        call_log = self.retrieve_call_log()
        call_duration = {}
        call_count = {}
        for k,v in call_log.items():
            if call_duration.get(v.get('number')):
                call_duration[v.get('number')] += int(v.get('duration'))
            else:
                call_duration[v.get('number')] = int(v.get('duration'))
            if call_count.get(v.get('number')):
                call_count[v.get('number')] += 1
            else:
                call_count[v.get('number')] = 1
        call_duration = sorted(call_duration.items(), key=lambda x: x[1], reverse=True)
        call_count = sorted(call_count.items(), key=lambda x: x[1], reverse=True)
        return {
            'longest_caller' : call_duration[:5],
            'frequent_caller' : call_count[:5]
        }
    def retrieve_sms_log(self):
        return Analytics.__parse_data_to_dic(self.adb.shell('content query --uri content://sms'))
    
    def retrieve_screen_time(self):
        return self.adb.shell('dumpsys usagestats')
    
    def analyse_screen_time(self):
        '''Analyse the screen time and return a dictionary with the following keys
        - most used app
        - most used app duration
        - maximum app duration'''
        screen_time = self.retrieve_screen_time()
        screen_time = screen_time.split('\n')


from advance_adb.adb import Adb

class files:
    def __init__(self, device_id=None):
        self.adb = Adb(device_id)
        
    def push(self, file_path, device_path):
        return self.adb.shell_raw(f'push {file_path} {device_path}')
    
    def pull(self, device_path, file_path):
        return self.adb.shell_raw(f'pull {device_path} {file_path}')
    
    def list_files(self, path):
        return self.adb.shell(f'ls {path}')
    
    def delete_file(self, file_path):
        return self.adb.shell(f'rm {file_path}')
    
    def delete_dir(self, dir_path):
        return self.adb.shell(f'rm -r {dir_path}')
    
    def make_dir(self, dir_path):
        return self.adb.shell(f'mkdir {dir_path}')
    
    def rename(self, old_path, new_path):
        return self.adb.shell(f'mv {old_path} {new_path}')
    
    def copy(self, src_path, dest_path):
        return self.adb.shell(f'cp {src_path} {dest_path}')
    
    def re_list_files(self, path):
        return self.adb.shell(f'ls -R {path}')


from advance_adb.adb import Adb
from advance_adb.files import files
from advance_adb.app import app
from advance_adb.misc import Misc
from advance_adb.utilities import utils
from advance_adb.analytics import Analytics
from advance_adb.whatsapp import whatsapp


from advance_adb.adb import Adb
from advance_adb.files import files
class Misc:
    def __init__(self, device_id=None):
        self.adb = Adb(device_id)
        self.file = files(device_id)
    def get_battery_level(self):
        return self.adb.shell('dumpsys battery | grep level')
    
    def get_battery_status(self):
        return self.adb.shell('dumpsys battery | grep status')
    
    def screenshot(self, file_path='/sdcard/screenshot.png'):
        return self.adb.shell(f'screencap {file_path}')
    
    def screenrecord(self, file_path='/sdcard/screenrecord.mp4', time_limit=180):
        return self.adb.shell(f'screenrecord --time-limit {time_limit} {file_path}')
    
    def pull_screenshot(self, file_path):
        return self.file.pull('/sdcard/screenshot.png', file_path)
    
    def pull_screenrecord(self, file_path):
        return self.file.pull('/sdcard/screenrecord.mp4', file_path)
    
    def get_screen_size(self):
        return self.adb.shell('wm size')
    
    def get_screen_density(self):
        return self.adb.shell('wm density')
    
    def get_screen_orientation(self):
        return self.adb.shell('dumpsys input | grep SurfaceOrientation')
    
    def open_url(self, url):
        return self.adb.shell(f'am start -a android.intent.action.VIEW -d {url}')
    


from advance_adb.adb import Adb

class app:
    def __init__(self, device_id=None):
        self.adb = Adb(device_id)
    def install(self, apk_path):
        return self.adb.shell_raw(f'install -r {apk_path}')
    
    def uninstall(self, package_name):
        return self.adb.shell_raw(f'uninstall {package_name}')
    
    def clear_data(self, package_name):
        return self.adb.shell(f'pm clear {package_name}')
    
    def launch_app(self, package_name):
        return self.adb.shell(f'monkey -p {package_name} 1')
    
    def force_stop(self, package_name):
        return self.adb.shell(f'am force-stop {package_name}')
    
    def start_activity(self, activity):
        return self.adb.shell(f'am start -n {activity}')
    
    def stop_activity(self, activity):
        return self.adb.shell(f'am force-stop {activity}')
    
    def get_current_activity(self):
        return self.adb.shell('dumpsys window windows | grep mCurrentFocus')
    
    def get_installed_packages(self):
        return self.adb.shell('pm list packages')
    
    def get_installed_packages_names(self):
        return [package.split(':')[1] for package in self.get_installed_packages().splitlines()]
    
    def disable(self, package_name):
        return self.adb.shell('pm disable --user 0 ' + package_name)
    
    def enable(self, package_name):
        return self.adb.shell('pm enable ' + package_name)
    
    def is_package_installed(self, package_name):
        return package_name in self.get_installed_packages_names()
    
    def grant_permission(self, package_name, permission):
        return self.adb.shell(f'pm grant {package_name} {permission}')
    
    def revoke_permission(self, package_name, permission):
        return self.adb.shell(f'pm revoke {package_name} {permission}')
    
    def get_permissions(self, package_name):
        return self.adb.shell(f'pm dump {package_name} | grep permission')
    
    def if_launchable(self, package_name):
        return True if self.adb.shell(f'pm dump {package_name} | grep -A 1 android.intent.action.MAIN | grep android.intent.category.LAUNCHER') else False


import subprocess as sp
import advance_adb.Exceptions as ex

class Adb():
    def __init__(self, device_id=None):
        self.device_id = device_id
        
    @staticmethod
    def devices():
        return sp.run(['adb', 'devices'], capture_output=True, text=True).stdout.strip()
    
    @staticmethod
    def kill_server():
        return sp.run(['adb', 'kill-server'], capture_output=True, text=True).stdout.strip()

    @staticmethod
    def start_server():
        return sp.run(['adb', 'start-server'], capture_output=True, text=True).stdout.strip()
    
    @staticmethod
    def restart_server():
        Adb.kill_server()
        return Adb.start_server()
    
    def shell_raw(self, cmd):
        cmd2 = ['adb']
        if self.device_id:
            cmd2 += ['-s', self.device_id]
        cmd2 += cmd.split()
        try:
            result =  sp.run(cmd2, capture_output=True, text=True).stdout.strip()
        except ex.AdbError as e:
            raise ex.AdbError(e)
        return result
    
    def shell(self, cmd_2):
        cmd = ['adb']
        if self.device_id:
            cmd += ['-s', self.device_id]
        cmd += ['shell', cmd_2]
        try:
            result =  sp.run(cmd, capture_output=True, text=True).stdout.strip()
        except ex.AdbError as e:
            raise ex.AdbError(e)
        return result
    
    def getprop(self, key):
        return self.shell(f'getprop {key}').strip()
    
    def setprop(self, key, value):
        return self.shell(f'setprop {key} {value}').strip()
    
    def get_device_id(self):
        return self.getprop('ro.serialno')
    
    def get_android_version(self):
        return self.getprop('ro.build.version.release')
    
    def get_sdk_version(self):
        return self.getprop('ro.build.version.sdk')
    
    def get_model(self):
        return self.getprop('ro.product.model')
    
    def get_manufacturer(self):
        return self.getprop('ro.product.manufacturer')


import setuptools
    
setuptools.setup(
    name="advance_adb",
    version="0.0.2",
    author="Sudhir Kumar",
    author_email="sudhir9122kumar@gmail.com",
    description="Advance adb toolkit in one place",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
)